/*
I2C_Scanner_N5IB.ino
19 October 2019

This program (or code that looks like it) can be found in many places.
For example on the Arduino.cc forum. The original author is not known

somewhat modified by J. Giammanco, N5IB, to clarify 7 bit vs 8 bit nomenclature

This sketch tests for the standard 7-bit slave addresses
Devices with higher bit address might not be seen properly.

***Note on I2C addresses***
A 7-bit slave address is used, but an additional R/not-W bit is appended at the LSB position
to indicate a READ (bit=1) or WRITE (bit=0)operation, 
yielding the equivalent of a pair of sequential 8 bit address.

For example, one particular I2C device specifies that its 7-bit slave address can be either 076hex or 077hex
as determined by a hardware jumper. 
It is one of these 7-bit addresses which will be reported by this software

A different device specifies its addresses by including the R/not-W bit, yielding an apparent pair of 8-bit addresses
In the given example the READ address is specified as 0A3hex, and the WRITE as 0A2h
For this device the scanning software would report the 7-bit address: 051hex
note that shifting 051hex one bit to the left yields the aforementioned 0A2hex

*/

#include <Wire.h>      // invoke the One Wire library
byte error, address;
int nDevices;     
void setup()
{
  Wire.begin();// initialize for I2C bus communication  
  Serial.begin(9600);// be sure this rate matches in Serial Monitor window
  while (!Serial);             // Leonardo: wait for serial monitor
  Serial.println("\nI2C Scanner");
}
       
void loop()
{   
   Serial.println("Scanning...");//sign on message
   Serial.println();
     
   nDevices = 0;
   for(address = 1; address < 127; address++ )
   {
    
/* 
The I2C_scanner uses the return value of
the Write.endTransmisstion() to see if
a device did acknowledge to the address.
*/
   Wire.beginTransmission(address);
   error = Wire.endTransmission();
     
    if (error == 0)// valid acknowledgement from an I2C device
    {
      Serial.print("I2C device found at 7-bit slave address 0x");
      if (address<16)  Serial.print("0");
      Serial.print(address,HEX);
      Serial.print("   (");
      Serial.print(address, BIN);
      Serial.println("n binary)  ");
      Serial.println("    Where 'n' is an appended READ/NOT_WRITE bit, therefore:");
      byte I2C_Waddr = address << 1; // generate the two (READ and WRITE) 8-bit addresses
      byte I2C_Raddr = I2C_Waddr + 0x01;
      Serial.print("    The full 8-bit WRITE address is 0x");
      if (I2C_Waddr<16) Serial.print("0");
      Serial.print(I2C_Waddr,HEX);
      Serial.print("  and 8-bit READ address is 0x");
      if (I2C_Raddr<16) Serial.print("0");
      Serial.print(I2C_Raddr,HEX);          
      Serial.println();
      Serial.println();
               
      nDevices++;// bump the count of devices found
    }

    else if (error==4)     // something answered, but failed to properly communicate
    {
      Serial.print("Unknown error at 7-bit slave address 0x");
      if (address<16)  Serial.print("0");
      Serial.println(address,HEX);
    }    
  }
   
  if (nDevices == 0)
     Serial.println("No I2C Devices Found\n");
  else
     Serial.println("Done - Will re-scan in 5 seconds...\n");
     
  delay(5000);           // wait 5 seconds for next scan
}

