/*  LCD Character Display Test Sketch
 *  Jim Giammanco, N5IB
 *  16 February 2020
 */
 
#include <LiquidCrystal_I2C.h>
#define BACKLIGHT_PIN     3
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address

byte StartChar, m;

//**********************************************************************
void setup(){
  analogReference(EXTERNAL);  // setting required to use a precision band gap voltage reference (here 4.096V) for the ADC reference

  lcd.begin(20, 4);  // configure for 20 x 4 display

  // Switch on the backlight
  pinMode ( BACKLIGHT_PIN, OUTPUT );
  digitalWrite ( BACKLIGHT_PIN, HIGH );

  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  Serial.setTimeout(30000);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  lcd.clear();
  lcd.setCursor(7,1);    
  lcd.print("Hello !");
  delay(4000);
  lcd.clear();
  CustomChars();
}

//******************************************************************
void loop()
{
  lcd.clear(); StartChar=0;
  BigNums(StartChar,0); delay(1000);
  BigNums(StartChar,1); delay(1000);
  BigNums(StartChar,2); delay(1000);
  BigNums(StartChar,3); delay(1000); 
  BigNums(StartChar,4); delay(1000);   
  BigNums(StartChar,10); delay(2000);
  lcd.clear(); StartChar=0;
  BigNums(StartChar,5); delay(1000);
  BigNums(StartChar,6); delay(1000);
  BigNums(StartChar,7); delay(1000);
  BigNums(StartChar,8); delay(1000);
  BigNums(StartChar,9); delay(4000);   
}

//**************************************************************************
void CustomChars() // creates a set of custom characters to be used to make large size displays
{
  byte tempChar[8];

  tempChar[0]=B00001;
  tempChar[1]=B00011;   
  tempChar[2]=B00111; 
  tempChar[3]=B01111;
  tempChar[4]=B11111;  
  tempChar[5]=B11111;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(0, tempChar);

  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B11111; 
  tempChar[3]=B11111;
  tempChar[4]=B01111;  
  tempChar[5]=B00111;
  tempChar[6]=B00011;
  tempChar[7]=B00001;
  lcd.createChar(1, tempChar);  

  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B11111; 
  tempChar[3]=B11111;
  tempChar[4]=B00000;  
  tempChar[5]=B00000;
  tempChar[6]=B00000;
  tempChar[7]=B00000;
  lcd.createChar(2, tempChar); 
  
  tempChar[0]=B00000;
  tempChar[1]=B00000;   
  tempChar[2]=B00000; 
  tempChar[3]=B00000;
  tempChar[4]=B11111;  
  tempChar[5]=B11111;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(3, tempChar);  
  
  tempChar[0]=B00000;
  tempChar[1]=B00000;   
  tempChar[2]=B00000; 
  tempChar[3]=B00000;
  tempChar[4]=B00000;  
  tempChar[5]=B00000;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(4, tempChar);  
  
  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B00000; 
  tempChar[3]=B00000;
  tempChar[4]=B00000;  
  tempChar[5]=B00000;
  tempChar[6]=B00000;
  tempChar[7]=B00000;
  lcd.createChar(5, tempChar);  

  tempChar[0]=B10000;
  tempChar[1]=B11000;   
  tempChar[2]=B11100; 
  tempChar[3]=B11110;
  tempChar[4]=B11111;  
  tempChar[5]=B11111;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(6, tempChar);
    
  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B11111; 
  tempChar[3]=B11111;
  tempChar[4]=B11110;  
  tempChar[5]=B11100;
  tempChar[6]=B11000;
  tempChar[7]=B10000;
  lcd.createChar(7, tempChar);  
}

//***************************************************************************
void BigNums(byte nnC, byte mm)
{
   switch(mm) {
     case 0:
         lcd.setCursor(nnC,0);
         lcd.write(8); lcd.write(2); lcd.write(6);  // "8" means the same as "0" here..... don't ask  %&$#*!
         lcd.setCursor(nnC,1);
         lcd.write(0xFF); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(nnC,2);
         lcd.write(0xFF); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(1); lcd.write(3); lcd.write(7);
         StartChar=nnC+4;
         break;
                        
     case 1:
         lcd.setCursor(nnC,0);  
         lcd.write(8);
         lcd.setCursor(nnC,1);
         lcd.write(0xFF);
         lcd.setCursor(nnC,2);
         lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(0xFF);
         StartChar=nnC+2;
         break;

     case 2:
         lcd.setCursor(nnC,0);  
         lcd.write(2); lcd.write(2); lcd.write(0xFF);
         lcd.setCursor(nnC,1);
         lcd.write(0x20); lcd.write(3); lcd.write(2);
         lcd.setCursor(nnC,2);
         lcd.write(0xFF); lcd.write(0x20); lcd.write(0x20);
         lcd.setCursor(nnC,3);
         lcd.write(1); lcd.write(3); lcd.write(8);
         StartChar=nnC+4;     
         break;

     case 3:
         lcd.setCursor(nnC,0);  
         lcd.write(2); lcd.write(2); lcd.write(6);
         lcd.setCursor(nnC,1);
         lcd.write(0x20); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(nnC,2);
         lcd.write(0x20); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(3); lcd.write(3); lcd.write(7);
         StartChar=nnC+4;     
         break;

     case 4:
         lcd.setCursor(nnC,0);  
         lcd.write(8); lcd.write(0x20); lcd.write(8);
         lcd.setCursor(nnC,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(nnC,2);
         lcd.write(5); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         StartChar=nnC+4;          
         break;

     case 5:
         lcd.setCursor(nnC,0);  
         lcd.write(8); lcd.write(2); lcd.write(2);
         lcd.setCursor(nnC,1);
         lcd.write(2); lcd.write(3); lcd.write(0x20);
         lcd.setCursor(nnC,2);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(6); lcd.write(3); lcd.write(7);
         StartChar=nnC+4;              
         break;

     case 6:
         lcd.setCursor(nnC,0);  
         lcd.write(8); lcd.write(2); lcd.write(2);
         lcd.setCursor(nnC,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(4);
         lcd.setCursor(nnC,2);
         lcd.write(0xFF); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(1); lcd.write(3); lcd.write(7);
         StartChar=nnC+4;         
         break;

     case 7:
         lcd.setCursor(nnC,0);  
         lcd.write(8); lcd.write(2); lcd.write(0xFF);
         lcd.setCursor(nnC,1);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(nnC,2);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(0x20); lcd.write(0x20); lcd.write(7);
         StartChar=nnC+4;              
         break;

     case 8:
         lcd.setCursor(nnC,0);  
         lcd.write(8); lcd.write(2); lcd.write(6);
         lcd.setCursor(nnC,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(nnC,2);
         lcd.write(0xFF); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(1); lcd.write(3); lcd.write(7);
         StartChar=nnC+4;              
         break; 

     case 9:
         lcd.setCursor(nnC,0);  
         lcd.write(8); lcd.write(2); lcd.write(6);
         lcd.setCursor(nnC,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(nnC,2);
         lcd.write(0x20); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(nnC,3);
         lcd.write(3); lcd.write(3); lcd.write(7);
         StartChar=nnC+4;                   
         break;

     case 10:  // draws ":"
         lcd.setCursor(nnC,1);  
         lcd.write(2);
         lcd.setCursor(nnC,2);
         lcd.write(3);
         StartChar=nnC+2;     
         break;                
   }  
}

